#version 330

uniform sampler2D uni_DiffuseMap;
uniform sampler2D uni_DiffuseMap2;
uniform sampler2D uni_SpecularMap;
uniform sampler2D uni_SpecularMap2;
uniform sampler2DShadow uni_ShadowMap;
uniform float uni_WaterHeight;
uniform vec3 uni_CameraPos;

uniform vec2 uni_ShadowSampleOffsets[25];

in vec4 var_WorldPosition;
in vec3 var_Normal;

in vec3 var_TextureLighting;
in vec3 var_SpecularLighting;

in vec4 var_PosViewSpace;
in vec4 var_SMTexCoords;

out vec4 frag_Color;
out vec4 frag_LSOcclusion;

// Shadow filter borders
#define SHADOW_BORDER_1 200
#define SHADOW_BORDER_2 400
#define SHADOW_BORDER_3 800
#define SHADOW_BORDER_4 1200


// do pcf based on fragment distance
// based on http://www.organicvectory.com/index.php?option=com_content&view=article&id=54:shadow-mapping-on-large-terrain&catid=1:articles&Itemid=61
float softShadow(float eyeDistance)
{
	float retVal = 0.0;

	vec3 shadowCoord = (var_SMTexCoords.xyz/var_SMTexCoords.w);
	vec3 coordOffset = shadowCoord;
	
	int idx = 0;
	if (eyeDistance < SHADOW_BORDER_1)
	{
		for (idx = 0; idx < 25; ++idx)
		{
			coordOffset.st = shadowCoord.st + uni_ShadowSampleOffsets[idx].st;
			retVal += texture(uni_ShadowMap, coordOffset) / float(25);
		}
	}
	else if (eyeDistance < SHADOW_BORDER_2)
	{
		for (idx = 0; idx < 16; ++idx)
		{
			coordOffset.st = shadowCoord.st + uni_ShadowSampleOffsets[idx].st;
			retVal += texture(uni_ShadowMap, coordOffset) / float(16);
		}
	}
	else if (eyeDistance < SHADOW_BORDER_3)
	{
		for (idx = 0; idx < 9; ++idx)
		{
			coordOffset.st = shadowCoord.st + uni_ShadowSampleOffsets[idx].st;
			retVal += texture(uni_ShadowMap, coordOffset) / float(9);
		}
	}
	else if (eyeDistance < SHADOW_BORDER_4)
	{
		for (idx = 0; idx < 4; idx++)
		{
			coordOffset.st = shadowCoord.st + uni_ShadowSampleOffsets[idx].st;
			retVal += texture(uni_ShadowMap, coordOffset) / float(4);
		}
	}
	else
	{
		retVal = (textureProj(uni_ShadowMap, var_SMTexCoords) < 1.0 ? 0.0 : 1.0);
	}
	
	if (retVal < 1.0) //if shadowed, do fade if necessary
	{
		float maxShadowRadius = 1400;
		
		float shadowFadeStart = maxShadowRadius * 0.9;
		
		if (eyeDistance > shadowFadeStart)
		{
			retVal = max((eyeDistance - shadowFadeStart)/
			(maxShadowRadius - shadowFadeStart),
			retVal);
		}
	}
	return clamp(retVal, 0.1, 1.0);
}

float shadowCoef(in vec4 texCoords)
{
	float zBias = 0.01;
	
	float shadowNess = 1-textureProj(uni_ShadowMap,texCoords);
	
	float distDampen = 1 - smoothstep(1200,1500,length(var_PosViewSpace.xyz));
	//vec2 dampenLow = smoothstep(0,0.1,texCoords.xy);
	//vec2 dampenHigh = 1 - smoothstep(0.9,1.0,texCoords.xy);
	//return 1-(shadowNess*dampenLow.x*dampenLow.y * dampenHigh.y * dampenHigh.x);
	//return texture(uni_ShadowMap,texCoords.xyz);
	return 1 - (shadowNess * distDampen);	
}

const mat4 colorMatrix = mat4(
   0.30, 0.0, 0.0, 0.0, // first column 
   0.0, 0.6, 0.2, 0.0, // second column
   0.3, 0.4, 0.8, 0.0, // third column
   0.0, 0.0, 0.0, 1.0 // fourth column
);

void main(void)
{
   vec3 blend_weights = abs(var_Normal);

   //blend_weights.xz *= 1.0f;
   blend_weights.y *= .65f;
   
   blend_weights = normalize(blend_weights);
   
   float tightBlending = 0.5f;
   blend_weights = blend_weights - tightBlending;
   blend_weights = max(blend_weights, 0.0001f);  //max should be always > 0, otherwise there are "black holes" in the textures
   
   // force sum to 1.0
   float sum = blend_weights.x + blend_weights.y + blend_weights.z;
   blend_weights = blend_weights.xyz / sum;

   float tex_scale = 0.01f;

   vec2 coord1 = var_WorldPosition.yz * tex_scale;
   vec2 coord2 = var_WorldPosition.zx * tex_scale;
   vec2 coord3 = var_WorldPosition.xy * tex_scale;

   //alpha is directly used for the specular maps weighting
   vec4 col1 = vec4(texture(uni_DiffuseMap2, coord1).rgb, texture(uni_SpecularMap2, coord1).r); 
   vec4 col2 = vec4(texture(uni_DiffuseMap, coord2).rgb, texture(uni_SpecularMap, coord2).r); 
   vec4 col3 = vec4(texture(uni_DiffuseMap2, coord3).rgb, texture(uni_SpecularMap2, coord3).r);

   //debug colorize - multiply instead of addition
   //col1 = vec4(1.0,0.0,0.0,1.0); 
   //col2 = vec4(0.0,1.0,0.0,1.0);
   //col3 = vec4(0.0,0.0,1.0,1.0);
   
   //col1.a = 0.1;
   //col3.a = 0.1;
   vec4 blended_color = col1.rgba * blend_weights.xxxx + 
						col2.rgba * blend_weights.yyyy + 
						col3.rgba * blend_weights.zzzz;

	//float shadow = shadowCoef(var_SMTexCoords);
	float shadow = softShadow(length(var_PosViewSpace.xyz));

    vec3 shadedColor = var_TextureLighting * blended_color.rgb  + var_SpecularLighting * blended_color.a;
	shadedColor *= shadow;
	
	float colorScale = 1.0f;
	if(uni_CameraPos.y>uni_WaterHeight)
		colorScale = smoothstep(uni_WaterHeight*0.95,uni_WaterHeight,var_WorldPosition.y);
	
	vec4 finalColor = vec4(shadedColor,1.0);
	vec4 correctedColor = colorMatrix * finalColor;
	
	 frag_Color = mix(correctedColor,finalColor,colorScale);
	//frag_Color = vec4(var_SMTexCoords.xy, 0.0, 1.0);
	//frag_Color = vec4(vec3(var_SMTexCoords.w*0.01),1.0);
    frag_LSOcclusion = vec4(0,0,0,1.0);
}